@@ -12,7 +12,6 @@
 #ifndef _BOOK_VIEW_H_
 #define _BOOK_VIEW_H_
 
-#include <glib-object.h>
 #include <gtk/gtk.h>
 #include <cairo.h>
 #include "book.h"
@@ -27,36 +26,28 @@ typedef struct BookViewPrivate BookViewPrivate;
 
 typedef struct
 {
-    GObject          parent_instance;
+    GtkVBox parent_instance;
     BookViewPrivate *priv;
 } BookView;
 
 typedef struct
 {
-    GObjectClass parent_class;
-    
+    GtkVBoxClass parent_class;
+
     void (*page_selected) (BookView *view, Page *page);
+    void (*show_page) (BookView *view, Page *page);
+    void (*show_menu) (BookView *view, Page *page);
 } BookViewClass;
 
 
 GType book_view_get_type (void);
 
-BookView *book_view_new (void);
+BookView *book_view_new (Book *book);
 
-GtkAdjustment *book_view_get_zoom_adjustment (BookView *view);
-
-// FIXME: Book view should extend GtkWidget
-void book_view_set_widget (BookView *view, GtkWidget *widget, GtkWidget *page_menu);
-
-// FIXME: Should be part of book_view_new
-void book_view_set_book (BookView *view, Book *book);
+void book_view_redraw (BookView *view);
 
 Book *book_view_get_book (BookView *view);
 
-void book_view_pan (BookView *view, gint x_offset, gint y_offset);
-
-void book_view_set_zoom (BookView *view, gdouble zoom);
-
 void book_view_select_page (BookView *view, Page *page);
 
 void book_view_select_next_page (BookView *view);
