@@ -457,6 +457,10 @@ set_default_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SA
 {
     SANE_Status status;
 
+    /* Check if supports automatic option */
+    if ((option->cap & SANE_CAP_AUTOMATIC) == 0)
+        return FALSE;
+
     status = sane_control_option (handle, option_index, SANE_ACTION_SET_AUTO, NULL, NULL);
     g_debug ("sane_control_option (%d, SANE_ACTION_SET_AUTO) -> %s",
              option_index, get_status_string (status));
@@ -788,7 +792,7 @@ close_device (Scanner *scanner)
         g_debug ("sane_close ()");
         scanner->priv->handle = NULL;
     }
-  
+
     g_free (scanner->priv->buffer);
     scanner->priv->buffer = NULL;
 
@@ -880,7 +884,7 @@ do_open (Scanner *scanner)
 
     if (!job->device) {
         g_warning ("No scan device available");
-        fail_scan (scanner, status,
+        fail_scan (scanner, 0,
                    /* Error displayed when no scanners to scan with */
                    _("No scanners available.  Please connect a scanner."));
         return;
@@ -986,6 +990,8 @@ do_get_option (Scanner *scanner)
     else if (strcmp (option->name, SANE_NAME_SCAN_SOURCE) == 0) {
         const char *flatbed_sources[] =
         {
+            "Auto",
+            SANE_I18N ("Auto"),
             "Flatbed",
             SANE_I18N ("Flatbed"),
             "FlatBed",
@@ -1048,6 +1054,14 @@ do_get_option (Scanner *scanner)
             break;
         }
     }
+    else if (strcmp (option->name, "duplex") == 0) {
+        if (option->type == SANE_TYPE_BOOL)
+            set_bool_option (scanner->priv->handle, option, option_index, job->type == SCAN_ADF_BOTH, NULL);
+    }
+    else if (strcmp (option->name, "batch-scan") == 0) {
+        if (option->type == SANE_TYPE_BOOL)
+            set_bool_option (scanner->priv->handle, option, option_index, job->type != SCAN_SINGLE, NULL);
+    }
     else if (strcmp (option->name, SANE_NAME_BIT_DEPTH) == 0) {
         if (job->depth > 0)
             set_int_option (scanner->priv->handle, option, option_index, job->depth, NULL);
@@ -1306,10 +1320,12 @@ do_read (Scanner *scanner)
                         n_to_read, &n_read);
     g_debug ("sane_read (%d) -> (%s, %d)", n_to_read, get_status_string (status), n_read);
 
-    /* End of variable length frame */
-    if (status == SANE_STATUS_EOF &&
-        scanner->priv->parameters.lines == -1 &&
-        scanner->priv->bytes_remaining == scanner->priv->parameters.bytes_per_line) {
+    /* Completed read */
+    if (status == SANE_STATUS_EOF) {
+        if (scanner->priv->parameters.lines > 0 && scanner->priv->line_count != scanner->priv->parameters.lines)
+            g_warning ("Scan completed with %d lines, expected %d lines", scanner->priv->parameters.lines, scanner->priv->parameters.lines);
+        if (scanner->priv->n_used > 0)
+            g_warning ("Scan complete with %d bytes of unused data", scanner->priv->n_used);
         do_complete_page (scanner);
         return;
     }
@@ -1330,6 +1346,7 @@ do_read (Scanner *scanner)
     /* Feed out lines */
     if (scanner->priv->n_used >= scanner->priv->parameters.bytes_per_line) {
         ScanLine *line;
+        int i, n_remaining;
 
         line = g_malloc(sizeof(ScanLine));
         switch (scanner->priv->parameters.format) {
@@ -1359,27 +1376,18 @@ do_read (Scanner *scanner)
         scanner->priv->buffer = NULL;
         scanner->priv->line_count += line->n_lines;
 
-        /* On last line */
-        if (scanner->priv->parameters.lines > 0 && scanner->priv->line_count >= scanner->priv->parameters.lines) {
-            emit_signal (scanner, GOT_LINE, line);
-            do_complete_page (scanner);
-        }
-        else {
-            int i, n_remaining;
+        /* Increase buffer size if did full read */
+        if (full_read)
+            scanner->priv->buffer_size += scanner->priv->parameters.bytes_per_line;
 
-            /* Increase buffer size if did full read */
-            if (full_read)
-                scanner->priv->buffer_size += scanner->priv->parameters.bytes_per_line;
-
-            scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * scanner->priv->buffer_size);
-            n_remaining = scanner->priv->n_used - (line->n_lines * line->data_length);
-            scanner->priv->n_used = 0;
-            for (i = 0; i < n_remaining; i++) {
-                scanner->priv->buffer[i] = line->data[i + (line->n_lines * line->data_length)];
-                scanner->priv->n_used++;                
-            }
-            emit_signal (scanner, GOT_LINE, line);
+        scanner->priv->buffer = g_malloc(sizeof(SANE_Byte) * scanner->priv->buffer_size);
+        n_remaining = scanner->priv->n_used - (line->n_lines * line->data_length);
+        scanner->priv->n_used = 0;
+        for (i = 0; i < n_remaining; i++) {
+            scanner->priv->buffer[i] = line->data[i + (line->n_lines * line->data_length)];
+            scanner->priv->n_used++;
         }
+        emit_signal (scanner, GOT_LINE, line);
     }
 }
 
