@@ -15,6 +15,7 @@
 #include <gtk/gtk.h>
 #include <unistd.h>
 #include <gudev/gudev.h>
+#include <dbus/dbus-glib.h>
 
 #include <sane/sane.h> // For SANE_STATUS_CANCELLED
 
@@ -23,7 +24,11 @@
 #include "book.h"
 
 
-static const char *default_device = NULL;
+static ScanDevice *default_device = NULL;
+
+static gboolean have_devices = FALSE;
+
+static GUdevClient *udev_client;
 
 static SimpleScan *ui;
 
@@ -41,7 +46,28 @@ static gboolean debug = FALSE;
 static void
 update_scan_devices_cb (Scanner *scanner, GList *devices)
 {
-    ui_set_scan_devices (ui, devices);
+    GList *devices_copy;
+  
+    devices_copy = g_list_copy (devices);
+  
+    /* If the default device is not detected add it to the list */
+    if (default_device) {
+        GList *i;
+
+        for (i = devices_copy; i; i = i->next) {
+            ScanDevice *device = i->data;
+            if (strcmp (device->name, default_device->name) == 0)
+                break;
+        }
+
+        if (!i)
+            devices_copy = g_list_prepend (devices_copy, default_device);
+    }
+
+    have_devices = devices_copy != NULL;
+    ui_set_scan_devices (ui, devices_copy);
+  
+    g_list_free (devices_copy);
 }
 
 
@@ -76,16 +102,16 @@ append_page ()
     /* Copy info from previous page */
     if (page) {
         orientation = page_get_orientation (page);
-        width = page_get_scan_width (page);
-        height = page_get_scan_height (page);
+        width = page_get_width (page);
+        height = page_get_height (page);
         dpi = page_get_dpi (page);
-        
+
         do_crop = page_has_crop (page);
         if (do_crop) {
             named_crop = page_get_named_crop (page);
             page_get_crop (page, &cx, &cy, &cw, &ch);
         }
-    }  
+    }
 
     page = book_append_page (book, width, height, dpi, orientation);
     if (do_crop) {
@@ -104,6 +130,71 @@ append_page ()
 }
 
 
+static void
+scanner_new_page_cb (Scanner *scanner)
+{
+    append_page ();
+}
+
+
+static gchar *
+get_profile_for_device (const gchar *current_device)
+{
+    gboolean ret;
+    DBusGConnection *connection;
+    DBusGProxy *proxy;
+    GError *error = NULL;
+    GType custom_g_type_string_string;
+    GPtrArray *profile_data_array = NULL;
+    gchar *device_id = NULL;
+    gchar *icc_profile = NULL;
+
+    /* Connect to the color manager on the session bus */
+    connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+    proxy = dbus_g_proxy_new_for_name (connection,
+                                       "org.gnome.ColorManager",
+                                       "/org/gnome/ColorManager",
+                                       "org.gnome.ColorManager");
+
+    /* Get color profile */
+    device_id = g_strdup_printf ("sane:%s", current_device);
+    custom_g_type_string_string = dbus_g_type_get_collection ("GPtrArray",
+                                                              dbus_g_type_get_struct("GValueArray",
+                                                                                     G_TYPE_STRING,
+                                                                                     G_TYPE_STRING,
+                                                                                     G_TYPE_INVALID));
+    ret = dbus_g_proxy_call (proxy, "GetProfilesForDevice", &error,
+                             G_TYPE_STRING, device_id,
+                             G_TYPE_STRING, "",
+                             G_TYPE_INVALID,
+                             custom_g_type_string_string, &profile_data_array,
+                             G_TYPE_INVALID);
+    g_object_unref (proxy);
+    g_free (device_id);
+    if (!ret) {
+        g_debug ("The request failed: %s", error->message);
+        g_error_free (error);
+        return NULL;
+    }
+
+    if (profile_data_array->len > 0) {
+        GValueArray *gva;
+        GValue *gv = NULL;
+
+        /* Just use the preferred profile filename */
+        gva = (GValueArray *) g_ptr_array_index (profile_data_array, 0);
+        gv = g_value_array_get_nth (gva, 1);
+        icc_profile = g_value_dup_string (gv);
+        g_value_unset (gv);
+    }
+    else
+        g_debug ("There are no ICC profiles for the device sane:%s", current_device);
+    g_ptr_array_free (profile_data_array, TRUE);
+
+    return icc_profile;
+}
+
+
 static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
@@ -113,8 +204,13 @@ scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
              info->width, info->height, info->depth);
 
     /* Add a new page */
-    page = append_page (FALSE);
+    page = append_page ();
     page_set_scan_area (page, info->width, info->height, info->dpi);
+
+    /* Get ICC color profile */
+    /* FIXME: The ICC profile could change */
+    /* FIXME: Don't do a D-bus call for each page, cache color profiles */
+    page_set_color_profile (page, get_profile_for_device (info->device));
 }
 
 
@@ -138,15 +234,7 @@ scanner_page_done_cb (Scanner *scanner)
 
 
 static void
-scanner_document_done_cb (Scanner *scanner)
-{
-    ui_set_scanning (ui, FALSE);
-    ui_set_have_scan (ui, TRUE);
-}
-
-
-static void
-scanner_failed_cb (Scanner *scanner, GError *error)
+remove_empty_page ()
 {
     Page *page;
 
@@ -157,54 +245,59 @@ scanner_failed_cb (Scanner *scanner, GError *error)
         page_finish (page);
     else
         book_delete_page (book, page); 
+}
 
+
+static void
+scanner_document_done_cb (Scanner *scanner)
+{
+    remove_empty_page ();
+}
+
+
+static void
+scanner_failed_cb (Scanner *scanner, GError *error)
+{
+    remove_empty_page ();
     if (!g_error_matches (error, SCANNER_TYPE, SANE_STATUS_CANCELLED)) {
         ui_show_error (ui,
                        /* Title of error dialog when scan failed */
                        _("Failed to scan"),
                        error->message,
-                       TRUE);
+                       have_devices);
     }
-        
-    ui_set_scanning (ui, FALSE);
-    ui_set_have_scan (ui, TRUE);
 }
 
 
 static void
-scan_cb (SimpleScan *ui, const gchar *device, gint dpi, const gchar *profile_name, gboolean continuous)
+scanner_scanning_changed_cb (Scanner *scanner)
 {
-    struct {
-        const gchar *name;
-        ScanMode mode;
-        const gchar *file_name;
-    } profiles[] = 
-    {
-        { "text", SCAN_MODE_LINEART,
-          /* Default name for PDF documents */
-          _("Scanned Document.pdf") },
-        { "photo", SCAN_MODE_COLOR,
-          /* Default name for JPEG documents */
-          _("Scanned Document.jpg") },
-        { NULL, SCAN_MODE_COLOR,
-          /* Default name for JPEG documents */
-          _("Scanned Document.jpg") }                
-    };
-    gint i;
+    ui_set_scanning (ui, scanner_is_scanning (scanner));
+}
 
-    g_debug ("Requesting scan of type %s at %d dpi from device '%s'", profile_name, dpi, device);
 
-    /* Find this profile */
-    for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
+static void
+scan_cb (SimpleScan *ui, const gchar *device, ScanOptions *options)
+{
+    /* Default filename to use when saving document (and extension will be added, e.g. .jpg) */
+    const gchar *filename_prefix = _("Scanned Document");
+    const gchar *extension;
+    gchar *filename;
+
+    if (options->scan_mode == SCAN_MODE_COLOR)
+        extension = "jpg";
+    else
+        extension = "pdf";
+
+    g_debug ("Requesting scan at %d dpi from device '%s'", options->dpi, device);
 
     if (!scanner_is_scanning (scanner))
         append_page ();
 
-    ui_set_have_scan (ui, FALSE);
-    ui_set_scanning (ui, TRUE);
- 
-    ui_set_default_file_name (ui, profiles[i].file_name);
-    scanner_scan (scanner, device, dpi, profiles[i].mode, 8, continuous);
+    filename = g_strdup_printf ("%s.%s", filename_prefix, extension);
+    ui_set_default_file_name (ui, filename);
+    g_free (filename);
+    scanner_scan (scanner, device, options);
 }
 
 
@@ -216,21 +309,25 @@ cancel_cb (SimpleScan *ui)
 
 
 static gboolean
-save_book (const gchar *uri, GError **error)
+save_book_by_extension (GFile *file, GError **error)
 {
     gboolean result;
-    gchar *uri_lower;
+    gchar *uri, *uri_lower;
 
+    uri = g_file_get_uri (file);
     uri_lower = g_utf8_strdown (uri, -1);
     if (g_str_has_suffix (uri_lower, ".pdf"))
-        result = book_save_pdf (book, uri, error);
+        result = book_save (book, "pdf", file, error);
     else if (g_str_has_suffix (uri_lower, ".ps"))
-        result = book_save_ps (book, uri, error);
+        result = book_save (book, "ps", file, error);
     else if (g_str_has_suffix (uri_lower, ".png"))
-        result = book_save_png (book, uri, error);
+        result = book_save (book, "png", file, error);
+    else if (g_str_has_suffix (uri_lower, ".tif") || g_str_has_suffix (uri_lower, ".tiff"))
+        result = book_save (book, "tiff", file, error);
     else
-        result = book_save_jpeg (book, uri, error);
+        result = book_save (book, "jpeg", file, error);
 
+    g_free (uri);
     g_free (uri_lower);
 
     return result;
@@ -241,80 +338,121 @@ static void
 save_cb (SimpleScan *ui, const gchar *uri)
 {
     GError *error = NULL;
+    GFile *file;
 
     g_debug ("Saving to '%s'", uri);
 
-    if (!save_book (uri, &error)) {
+    file = g_file_new_for_uri (uri);
+    if (!save_book_by_extension (file, &error)) {
         g_warning ("Error saving file: %s", error->message);
         ui_show_error (ui,
                        /* Title of error dialog when save failed */
                        _("Failed to save file"),
                        error->message,
-		       FALSE);
+                       FALSE);
         g_error_free (error);
     }
+    g_object_unref (file);
+}
+
+
+static gchar *
+get_temporary_filename (const gchar *prefix, const gchar *extension)
+{
+    gint fd;
+    gchar *filename, *path;
+    GError *error = NULL;
+
+    /* NOTE: I'm not sure if this is a 100% safe strategy to use g_file_open_tmp(), close and
+     * use the filename but it appears to work in practise */
+
+    filename = g_strdup_printf ("%sXXXXXX.%s", prefix, extension);
+    fd = g_file_open_tmp (filename, &path, &error);
+    g_free (filename);
+    if (fd < 0) {
+        g_warning ("Error saving email attachment: %s", error->message);
+        g_clear_error (&error);
+        return NULL;
+    }
+    close (fd);
+
+    return path;
 }
 
 
 static void
-email_cb (SimpleScan *ui)
+email_cb (SimpleScan *ui, const gchar *profile)
 {
-    gint i;
-    gchar *dir, *path = NULL, *uri = NULL;
+    gboolean saved = FALSE;
+    GError *error = NULL;
+    GString *command_line;
+  
+    command_line = g_string_new ("xdg-email");
 
-    // TODO: Delete old files on startup
+    /* Save text files as PDFs */
+    if (strcmp (profile, "text") == 0) {
+        gchar *path;
 
-    /* Save in the temporary dir */
-    dir = g_build_filename (g_get_user_cache_dir (), "simple-scan", "email", NULL);
-    g_mkdir_with_parents (dir, 0700);
+        /* Open a temporary file */
+        path = get_temporary_filename ("scan", "pdf");
+        if (path) {
+            GFile *file;
 
-    for (i = 0; ; i++) {
-        GString *filename;
-        GError *error = NULL;
-
-        filename = g_string_new ("");
-        g_string_printf (filename, "scan-%d-%d.pdf", getpid (), i);
-
-        g_free (path);
-        path = g_build_filename (dir, filename->str, NULL);
-        g_string_free (filename, TRUE);
-
-        g_free (uri);
-        uri = g_filename_to_uri (path, NULL, NULL);
-
-        if (book_save_pdf (book, uri, &error)) {
-            GString *command_line;
-
-            command_line = g_string_new ("");
-            g_string_printf (command_line, "xdg-email --attach %s", path);
-            g_debug ("Launchind email client: %s", command_line->str);
-            g_spawn_command_line_async (command_line->str, &error);
-
-            if (error) {
-                g_warning ("Unable to start email: %s", error->message);
-                g_clear_error (&error);
-            }
-            g_string_free (command_line, TRUE);
-	    break;
+            file = g_file_new_for_path (path);
+            saved = book_save (book, "pdf", file, &error);
+            g_string_append_printf (command_line, " --attach %s", path);
+            g_free (path);
+            g_object_unref (file);
         }
-        else {
-            if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-                g_warning ("Unable to save email PDF: %s", error->message);
-                g_clear_error (&error);
+    }
+    else {
+        gint i;
+
+        for (i = 0; i < book_get_n_pages (book); i++) {
+            gchar *path;
+            GFile *file;
+
+            path = get_temporary_filename ("scan", "jpg");
+            if (!path) {
+                saved = FALSE;
                 break;
             }
+
+            file = g_file_new_for_path (path);
+            saved = page_save (book_get_page (book, i), "jpeg", file, &error);
+            g_string_append_printf (command_line, " --attach %s", path);
+            g_free (path);
+            g_object_unref (file);
+          
+            if (!saved)
+                break;
         }
     }
 
-    g_free (path);
-    g_free (uri);   
-    g_free (dir);
+    if (saved) {
+        g_debug ("Launchind email client: %s", command_line->str);
+        g_spawn_command_line_async (command_line->str, &error);
+
+        if (error) {
+            g_warning ("Unable to start email: %s", error->message);
+            g_clear_error (&error);
+        }
+    }
+    else {
+        g_warning ("Unable to save email file: %s", error->message);
+        g_clear_error (&error);
+    }
+
+    g_string_free (command_line, TRUE);
 }
 
 
 static void
 quit_cb (SimpleScan *ui)
 {
+    g_object_unref (book);
+    g_object_unref (ui);
+    g_object_unref (udev_client);
     scanner_free (scanner);
     gtk_main_quit ();
 }
@@ -342,7 +480,7 @@ usage(int show_gtk)
     fprintf(stderr,
             /* Description on how to use simple-scan displayed on command-line */    
             _("Help Options:\n"
-              "  -d, --debug                     Print debugging messages\n"	      
+              "  -d, --debug                     Print debugging messages\n"
               "  -v, --version                   Show release version\n"
               "  -h, --help                      Show help options\n"
               "  --help-all                      Show all help options\n"
@@ -443,7 +581,9 @@ get_options (int argc, char **argv)
                 fprintf (stderr, "Unknown argument: '%s'\n", arg);
                 exit (1);
             }
-            default_device = arg;
+            default_device = g_malloc0 (sizeof (ScanDevice));
+            default_device->name = g_strdup (arg);
+            default_device->label = g_strdup (arg);
         }
     }   
 }
@@ -455,10 +595,10 @@ on_uevent (GUdevClient *client, const gchar *action, GUdevDevice *device)
     scanner_redetect (scanner);
 }
 
+
 int
 main (int argc, char **argv)
 {
-    GUdevClient *udev_client;
     const char *udev_subsystems[] = { "usb", NULL };
     gchar *path;
 
@@ -495,17 +635,26 @@ main (int argc, char **argv)
     scanner = scanner_new ();
     g_signal_connect (G_OBJECT (scanner), "update-devices", G_CALLBACK (update_scan_devices_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "authorize", G_CALLBACK (authorize_cb), NULL);
+    g_signal_connect (G_OBJECT (scanner), "expect-page", G_CALLBACK (scanner_new_page_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-page-info", G_CALLBACK (scanner_page_info_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-line", G_CALLBACK (scanner_line_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "page-done", G_CALLBACK (scanner_page_done_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "document-done", G_CALLBACK (scanner_document_done_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "scan-failed", G_CALLBACK (scanner_failed_cb), NULL);
+    g_signal_connect (G_OBJECT (scanner), "scanning-changed", G_CALLBACK (scanner_scanning_changed_cb), NULL);
 
     udev_client = g_udev_client_new (udev_subsystems);
     g_signal_connect (udev_client, "uevent", G_CALLBACK (on_uevent), NULL);
 
-    if (default_device)
-        ui_set_selected_device (ui, default_device);
+    if (default_device) {
+        GList device_list;
+
+        device_list.data = default_device;
+        device_list.next = NULL;
+        device_list.prev = NULL;
+        ui_set_scan_devices (ui, &device_list);
+        ui_set_selected_device (ui, default_device->name);
+    }
 
     ui_start (ui);
     scanner_start (scanner);
