@@ -39,32 +39,38 @@ static guint signals[LAST_SIGNAL] = { 0, };
 struct SimpleScanPrivate
 {
     GConfClient *client;
-    
+
     GtkBuilder *builder;
 
-    GtkWidget *window;
-    GtkWidget *mode_combo;
-    GtkTreeModel *mode_model;
-    GtkWidget *preview_box, *preview_area, *preview_scroll;
+    GtkWidget *window, *main_vbox;
+    GtkWidget *info_bar, *info_bar_image, *info_bar_label;
+    GtkWidget *info_bar_close_button, *info_bar_change_scanner_button;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
     GtkWidget *stop_menuitem, *stop_toolbutton;
 
+    GtkWidget *text_toolbar_menuitem, *text_menu_menuitem;
+    GtkWidget *photo_toolbar_menuitem, *photo_menu_menuitem;
+
     GtkWidget *authorize_dialog;
     GtkWidget *authorize_label;
     GtkWidget *username_entry, *password_entry;
 
     GtkWidget *preferences_dialog;
-    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo, *page_side_combo;
-    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model, *page_side_model;
+    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo, *page_side_combo, *paper_size_combo;
+    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model, *page_side_model, *paper_size_model;
     gboolean setting_devices, user_selected_device;
 
+    gboolean have_error;
+    gchar *error_title, *error_text;
+    gboolean error_change_scanner_hint;
+
     Book *book;
     BookView *book_view;
     gboolean updating_page_menu;
     gint default_page_width, default_page_height, default_page_dpi;
     Orientation default_page_orientation;
   
-    gboolean have_device_list;
+    gchar *document_hint;
 
     gchar *default_file_name;
     gboolean scanning;
@@ -108,6 +114,22 @@ find_scan_device (SimpleScan *ui, const char *device, GtkTreeIter *iter)
 }
 
 
+static void
+show_error_dialog (SimpleScan *ui, const char *error_title, const char *error_text)
+{
+    GtkWidget *dialog;
+
+    dialog = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_WARNING,
+                                     GTK_BUTTONS_NONE,
+                                     "%s", error_title);
+   gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, 0);
+   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error_text);
+   gtk_widget_destroy (dialog);
+}
+
+
 void
 ui_set_default_file_name (SimpleScan *ui, const gchar *default_file_name)
 {
@@ -152,6 +174,47 @@ device_combo_changed_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+static void
+update_info_bar (SimpleScan *ui)
+{
+    GtkMessageType type;
+    const gchar *title, *text, *image_id;
+    gchar *message;
+    gboolean show_close_button = FALSE;
+    gboolean show_change_scanner_button = FALSE;
+  
+    if (ui->priv->have_error)  {
+        type = GTK_MESSAGE_ERROR;
+        image_id = GTK_STOCK_DIALOG_ERROR;
+        title = ui->priv->error_title;
+        text = ui->priv->error_text;
+        show_close_button = TRUE;
+        show_change_scanner_button = ui->priv->error_change_scanner_hint;
+    }
+    else if (gtk_tree_model_iter_n_children (ui->priv->device_model, NULL) == 0) {
+        type = GTK_MESSAGE_WARNING;
+        image_id = GTK_STOCK_DIALOG_WARNING;
+        /* Warning displayed when no scanners are detected */
+        title = _("No scanners detected");
+        /* Hint to user on why there are no scanners detected */
+        text = _("Please check your scanner is connected and powered on");
+    }
+    else {
+        gtk_widget_hide (ui->priv->info_bar);
+        return;
+    }
+
+    gtk_info_bar_set_message_type (GTK_INFO_BAR (ui->priv->info_bar), type);
+    gtk_image_set_from_stock (GTK_IMAGE (ui->priv->info_bar_image), image_id, GTK_ICON_SIZE_DIALOG);
+    message = g_strdup_printf ("<big><b>%s</b></big>\n\n%s", title, text);
+    gtk_label_set_markup (GTK_LABEL (ui->priv->info_bar_label), message);
+    g_free (message);
+    gtk_widget_set_visible (ui->priv->info_bar_close_button, show_close_button);  
+    gtk_widget_set_visible (ui->priv->info_bar_change_scanner_button, show_change_scanner_button);
+    gtk_widget_show (ui->priv->info_bar);
+}
+
+
 void
 ui_set_scan_devices (SimpleScan *ui, GList *devices)
 {
@@ -195,6 +258,9 @@ ui_set_scan_devices (SimpleScan *ui, GList *devices)
         if (n_delete >= 0) {
             gint i;
 
+            /* Update label */
+            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 1, device->label, -1);
+
             for (i = 0; i < n_delete; i++) {
                 gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index);
                 gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
@@ -217,18 +283,7 @@ ui_set_scan_devices (SimpleScan *ui, GList *devices)
 
     ui->priv->setting_devices = FALSE;
 
-    if (!ui->priv->have_device_list) {
-        ui->priv->have_device_list = TRUE;
-
-        if (!devices) {
-            ui_show_error (ui,
-                           /* Warning displayed when no scanners are detected */
-                           _("No scanners detected"),
-                           /* Hint to user on why there are no scanners detected */
-                           _("Please check your scanner is connected and powered on"),
-                           FALSE);
-        }
-    }
+    update_info_bar (ui);
 }
 
 
@@ -252,13 +307,11 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
 {
     GtkTreeIter iter;
 
-    /* If doesn't exist add with label set to device name */
-    if (!find_scan_device (ui, device, &iter)) {
-        gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-        gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device, 1, device, -1);
-    }
+    if (!find_scan_device (ui, device, &iter))
+        return;
 
     gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
+    ui->priv->user_selected_device = TRUE;
 }
 
 
@@ -266,9 +319,10 @@ static void
 add_default_page (SimpleScan *ui)
 {
     Page *page;
-  
+
     page = book_append_page (ui->priv->book,
-                             ui->priv->default_page_width, ui->priv->default_page_height,
+                             ui->priv->default_page_width,
+                             ui->priv->default_page_height,
                              ui->priv->default_page_dpi,
                              ui->priv->default_page_orientation);
     book_view_select_page (ui->priv->book_view, page);
@@ -288,23 +342,37 @@ new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 static void
 set_document_hint (SimpleScan *ui, const gchar *document_hint)
 {
-    GtkTreeIter iter;
+    g_free (ui->priv->document_hint);
+    ui->priv->document_hint = g_strdup (document_hint);
 
-    if (gtk_tree_model_get_iter_first (ui->priv->mode_model, &iter)) {
-        do {
-            gchar *d;
-            gboolean have_match;
+    if (strcmp (document_hint, "text") == 0) {
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->text_toolbar_menuitem), TRUE);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->text_menu_menuitem), TRUE);
+    }
+    else if (strcmp (document_hint, "photo") == 0) {
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->photo_toolbar_menuitem), TRUE);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->photo_menu_menuitem), TRUE);
+    }
+}
 
-            gtk_tree_model_get (ui->priv->mode_model, &iter, 0, &d, -1);
-            have_match = strcmp (d, document_hint) == 0;
-            g_free (d);
 
-            if (have_match) {
-                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->mode_combo), &iter);                
-                return;
-            }
-        } while (gtk_tree_model_iter_next (ui->priv->mode_model, &iter));
-     }
+void text_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+text_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_document_hint (ui, "text");
+}
+
+
+void photo_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+photo_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_document_hint (ui, "photo");
 }
 
 
@@ -331,6 +399,29 @@ set_page_side (SimpleScan *ui, const gchar *document_hint)
 }
 
 
+static void
+set_paper_size (SimpleScan *ui, gint width, gint height)
+{
+    GtkTreeIter iter;
+    gboolean have_iter;
+  
+    for (have_iter = gtk_tree_model_get_iter_first (ui->priv->paper_size_model, &iter);
+         have_iter;
+         have_iter = gtk_tree_model_iter_next (ui->priv->paper_size_model, &iter)) {
+        gint w, h;
+
+        gtk_tree_model_get (ui->priv->paper_size_model, &iter, 0, &w, 1, &h, -1);
+        if (w == width && h == height)
+            break;
+    }
+  
+    if (!have_iter)
+        have_iter = gtk_tree_model_get_iter_first (ui->priv->paper_size_model, &iter);
+    if (have_iter)
+        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->paper_size_combo), &iter);
+}
+
+
 static gint
 get_text_dpi (SimpleScan *ui)
 {
@@ -370,16 +461,49 @@ get_page_side (SimpleScan *ui)
 }
 
 
-static gchar *
-get_document_hint (SimpleScan *ui)
+static gboolean
+get_paper_size (SimpleScan *ui, gint *width, gint *height)
 {
     GtkTreeIter iter;
-    gchar *mode = NULL;
 
-    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->mode_combo), &iter))
-        gtk_tree_model_get (ui->priv->mode_model, &iter, 0, &mode, -1);
-    
-    return mode;
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->paper_size_combo), &iter)) {
+        gtk_tree_model_get (ui->priv->paper_size_model, &iter, 0, width, 1, height, -1);
+        return TRUE;
+    }
+  
+    return FALSE;
+}
+
+
+static ScanOptions *
+get_scan_options (SimpleScan *ui)
+{
+    struct {
+        const gchar *name;
+        ScanMode mode;
+    } profiles[] =
+    {
+        { "text",  SCAN_MODE_LINEART },
+        { "photo", SCAN_MODE_COLOR   },
+        { NULL,    SCAN_MODE_COLOR   }
+    };
+    gint i;
+    ScanOptions *options;
+
+    /* Find this profile */
+    // FIXME: Move this into scan-profile.c
+    for (i = 0; profiles[i].name && strcmp (profiles[i].name, ui->priv->document_hint) != 0; i++);
+  
+    options = g_malloc0 (sizeof (ScanOptions));
+    options->scan_mode = profiles[i].mode;
+    options->depth = 8;
+    if (options->scan_mode == SCAN_MODE_COLOR)
+        options->dpi = get_photo_dpi (ui);
+    else
+        options->dpi = get_text_dpi (ui);
+    get_paper_size (ui, &options->paper_width, &options->paper_height);
+  
+    return options;
 }
 
 
@@ -388,18 +512,16 @@ G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gchar *device, *mode;
-    gint dpi;
+    gchar *device;
+    ScanOptions *options;
 
     device = get_selected_device (ui);
-    mode = get_document_hint (ui);
-    if (strcmp (mode, "text") == 0) 
-        dpi = get_text_dpi (ui);
-    else
-        dpi = get_photo_dpi (ui);
-    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, SCAN_SINGLE);
+
+    options = get_scan_options (ui);
+    options->type = SCAN_SINGLE;
+    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, options);
     g_free (device);
-    g_free (mode);
+    g_free (options);
 }
 
 
@@ -420,26 +542,23 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     if (ui->priv->scanning) {
         g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
     } else {
-        gchar *device, *mode, *side;
-        gint dpi;
-        ScanType type = SCAN_ADF_BOTH;
+        gchar *device, *side;
+        ScanOptions *options;
 
         device = get_selected_device (ui);
-        mode = get_document_hint (ui);
+        options = get_scan_options (ui);
         side = get_page_side (ui);
-        if (strcmp (mode, "text") == 0) 
-            dpi = get_text_dpi (ui);
-        else
-            dpi = get_photo_dpi (ui);
         if (strcmp (side, "front") == 0)
-            type = SCAN_ADF_FRONT;
+            options->type = SCAN_ADF_FRONT;
         else if (strcmp (side, "back") == 0)
-            type = SCAN_ADF_BACK;
+            options->type = SCAN_ADF_BACK;
+        else
+            options->type = SCAN_ADF_BOTH;
 
-        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, type);
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, options);
         g_free (device);
-        g_free (mode);
         g_free (side);
+        g_free (options);
     }
 }
 
@@ -484,7 +603,7 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
     if (page_has_crop (page)) {
         char *crop_name;
       
-        // FIXME: Make more generic
+        // FIXME: Make more generic, move into page-size.c and reuse
         crop_name = page_get_named_crop (page);
         if (crop_name) {
             if (strcmp (crop_name, "A4") == 0)
@@ -542,31 +661,42 @@ get_temporary_filename (const gchar *prefix, const gchar *extension)
 static void
 show_page_cb (BookView *view, Page *page, SimpleScan *ui)
 {
-    gchar *path, *uri;
+    gchar *path;
+    GFile *file;
     GdkScreen *screen;
     GError *error = NULL;
   
     path = get_temporary_filename ("scanned-page", "tiff");
     if (!path)
         return;
+    file = g_file_new_for_path (path);
+    g_free (path);
 
     screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
 
-    uri = g_filename_to_uri (path, NULL, NULL);
-    if (page_save_tiff (page, uri, &error))
+    if (page_save (page, "tiff", file, &error)) {
+        gchar *uri = g_file_get_uri (file);
         gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
-
-    if (error) {
-        ui_show_error (ui,
-                       /* Error message display when unable to preview image */
-                       _("Unable to open image preview application"),
-                       error->message,
-                       FALSE);
-        g_clear_error (&error);
+        g_free (uri);
     }
 
-    g_free (uri);
-    g_free (path);
+    g_object_unref (file);
+
+    if (error) {
+        show_error_dialog (ui,
+                           /* Error message display when unable to preview image */
+                           _("Unable to open image preview application"),
+                           error->message);
+        g_clear_error (&error);
+    }
+}
+
+
+static void
+show_page_menu_cb (BookView *view, SimpleScan *ui)
+{
+    gtk_menu_popup (GTK_MENU (gtk_builder_get_object (ui->priv->builder, "page_menu")), NULL, NULL, NULL, NULL,
+                    3, gtk_get_current_event_time());
 }
 
 
@@ -945,10 +1075,7 @@ G_MODULE_EXPORT
 void
 email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gchar *mode;
-    mode = get_document_hint (ui);
-    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, mode);
-    g_free (mode);
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, ui->priv->document_hint);
 }
 
 
@@ -985,11 +1112,10 @@ help_contents_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 
     if (error)
     {
-        ui_show_error (ui,
-                       /* Error message displayed when unable to launch help browser */
-                       _("Unable to open help file"),
-                       error->message,
-                      FALSE);
+        show_error_dialog (ui,
+                           /* Error message displayed when unable to launch help browser */
+                           _("Unable to open help file"),
+                           error->message);
         g_clear_error (&error);
     }
 }
@@ -1041,7 +1167,8 @@ about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 static void
 quit (SimpleScan *ui)
 {
-    char *device, *document_type;
+    char *device;
+    gint paper_width = 0, paper_height = 0;
     gint i;
 
     // FIXME: Warn if document with unsaved changes
@@ -1052,12 +1179,13 @@ quit (SimpleScan *ui)
         g_free (device);
     }
 
-    document_type = get_document_hint (ui);
-    gconf_client_set_string(ui->priv->client, GCONF_DIR "/document_type", document_type, NULL);
-    g_free (document_type);
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/document_type", ui->priv->document_hint, NULL);
     gconf_client_set_int (ui->priv->client, GCONF_DIR "/text_dpi", get_text_dpi (ui), NULL);
     gconf_client_set_int (ui->priv->client, GCONF_DIR "/photo_dpi", get_photo_dpi (ui), NULL);
     gconf_client_set_string (ui->priv->client, GCONF_DIR "/page_side", get_page_side (ui), NULL);
+    get_paper_size (ui, &paper_width, &paper_height);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/paper_width", paper_width, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/paper_height", paper_height, NULL);
 
     gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_width", ui->priv->window_width, NULL);
     gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_height", ui->priv->window_height, NULL);
@@ -1097,6 +1225,24 @@ simple_scan_window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *eve
 }
 
 
+static void
+info_bar_response_cb (GtkWidget *widget, gint response_id, SimpleScan *ui)
+{
+    if (response_id == 1) {
+        gtk_widget_grab_focus (ui->priv->device_combo);
+        gtk_window_present (GTK_WINDOW (ui->priv->preferences_dialog));
+    }
+    else {
+        ui->priv->have_error = FALSE;
+        g_free (ui->priv->error_title);
+        ui->priv->error_title = NULL;
+        g_free (ui->priv->error_text);
+        ui->priv->error_text = NULL;
+        update_info_bar (ui);      
+    }
+}
+
+
 gboolean simple_scan_window_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, SimpleScan *ui);
 G_MODULE_EXPORT
 gboolean
@@ -1121,8 +1267,8 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 static void
 page_size_changed_cb (Page *page, SimpleScan *ui)
 {
-    ui->priv->default_page_width = page_get_scan_width (page);
-    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_width = page_get_width (page);
+    ui->priv->default_page_height = page_get_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
 }
 
@@ -1137,8 +1283,8 @@ page_orientation_changed_cb (Page *page, SimpleScan *ui)
 static void
 page_added_cb (Book *book, Page *page, SimpleScan *ui)
 {
-    ui->priv->default_page_width = page_get_scan_width (page);
-    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_width = page_get_width (page);
+    ui->priv->default_page_height = page_get_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
     ui->priv->default_page_orientation = page_get_orientation (page);
     g_signal_connect (page, "size-changed", G_CALLBACK (page_size_changed_cb), ui);
@@ -1214,9 +1360,10 @@ ui_load (SimpleScan *ui)
 {
     GtkBuilder *builder;
     GError *error = NULL;
+    GtkWidget *hbox;
     GtkCellRenderer *renderer;
     gchar *device, *document_type, *scan_direction, *page_side;
-    gint dpi;
+    gint dpi, paper_width, paper_height;
 
     gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICON_DIR);
 
@@ -1226,31 +1373,26 @@ ui_load (SimpleScan *ui)
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
     if (error) {
         g_critical ("Unable to load UI: %s\n", error->message);
-        ui_show_error (ui,
-                       /* Title of dialog when cannot load required files */
-                       _("Files missing"),
-                       /* Description in dialog when cannot load required files */
-                       _("Please check your installation"),
-                       FALSE);
+        show_error_dialog (ui,
+                           /* Title of dialog when cannot load required files */
+                           _("Files missing"),
+                           /* Description in dialog when cannot load required files */
+                           _("Please check your installation"));
         exit (1);
     }
     gtk_builder_connect_signals (builder, ui);
 
     ui->priv->window = GTK_WIDGET (gtk_builder_get_object (builder, "simple_scan_window"));
-    ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
-    ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
-    ui->priv->preview_box = GTK_WIDGET (gtk_builder_get_object (builder, "preview_vbox"));
-    ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
-    ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
+    ui->priv->main_vbox = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
     ui->priv->stop_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "stop_scan_menuitem"));
     ui->priv->stop_toolbutton = GTK_WIDGET (gtk_builder_get_object (builder, "stop_toolbutton"));
 
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, "text", 1);
-    gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->mode_combo), 0);
+    ui->priv->text_toolbar_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "text_toolbutton_menuitem"));
+    ui->priv->text_menu_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "text_menuitem"));
+    ui->priv->photo_toolbar_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "photo_toolbutton_menuitem"));
+    ui->priv->photo_menu_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "photo_menuitem"));
 
     ui->priv->authorize_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_dialog"));
     ui->priv->authorize_label = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_label"));
@@ -1266,6 +1408,48 @@ ui_load (SimpleScan *ui)
     ui->priv->photo_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->photo_dpi_combo));
     ui->priv->page_side_combo = GTK_WIDGET (gtk_builder_get_object (builder, "page_side_combo"));
     ui->priv->page_side_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->page_side_combo));
+    ui->priv->paper_size_combo = GTK_WIDGET (gtk_builder_get_object (builder, "paper_size_combo"));
+    ui->priv->paper_size_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->paper_size_combo));
+
+    /* Add InfoBar (not supported in Glade) */
+    ui->priv->info_bar = gtk_info_bar_new ();
+    g_signal_connect (ui->priv->info_bar, "response", G_CALLBACK (info_bar_response_cb), ui);  
+    gtk_box_pack_start (GTK_BOX(ui->priv->main_vbox), ui->priv->info_bar, FALSE, TRUE, 0);
+    hbox = gtk_hbox_new (FALSE, 12);
+    gtk_container_add (GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (ui->priv->info_bar))), hbox);
+    gtk_widget_show (hbox);
+
+    ui->priv->info_bar_image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+    gtk_box_pack_start (GTK_BOX(hbox), ui->priv->info_bar_image, FALSE, TRUE, 0);
+    gtk_widget_show (ui->priv->info_bar_image);
+
+    ui->priv->info_bar_label = gtk_label_new (NULL);
+    gtk_misc_set_alignment (GTK_MISC (ui->priv->info_bar_label), 0.0, 0.5);
+    gtk_box_pack_start (GTK_BOX(hbox), ui->priv->info_bar_label, TRUE, TRUE, 0);
+    gtk_widget_show (ui->priv->info_bar_label);
+
+    ui->priv->info_bar_close_button = gtk_info_bar_add_button (GTK_INFO_BAR (ui->priv->info_bar), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+    ui->priv->info_bar_change_scanner_button = gtk_info_bar_add_button (GTK_INFO_BAR (ui->priv->info_bar),
+                                                                        /* Button in error infobar to open preferences dialog and change scanner */
+                                                                        _("Change _Scanner"), 1);
+
+    GtkTreeIter iter;
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 0, 1, 0, 2,
+                        /* Combo box value for automatic paper size */
+                        _("Automatic"), -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 1050, 1, 1480, 2, "A6", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 1480, 1, 2100, 2, "A5", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2100, 1, 2970, 2, "A4", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2159, 1, 2794, 2, "Letter", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 2159, 1, 3556, 2, "Legal", -1);
+    gtk_list_store_append (GTK_LIST_STORE (ui->priv->paper_size_model), &iter);
+    gtk_list_store_set (GTK_LIST_STORE (ui->priv->paper_size_model), &iter, 0, 1016, 1, 1524, 2, "4×6", -1);
 
     dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/text_dpi", NULL);
     if (dpi <= 0)
@@ -1279,12 +1463,23 @@ ui_load (SimpleScan *ui)
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, "text", 1);
+
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, "text", 1);
     page_side = gconf_client_get_string (ui->priv->client, GCONF_DIR "/page_side", NULL);
     if (page_side) {
         set_page_side (ui, page_side);
         g_free (page_side);
     }
 
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->paper_size_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->paper_size_combo), renderer, "text", 2);
+    paper_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/paper_width", NULL);
+    paper_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/paper_height", NULL);
+    set_paper_size (ui, paper_width, paper_height);
+
     device = gconf_client_get_string (ui->priv->client, GCONF_DIR "/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
@@ -1293,24 +1488,19 @@ ui_load (SimpleScan *ui)
         g_free (device);
     }
 
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, "text", 1);
-
     document_type = gconf_client_get_string (ui->priv->client, GCONF_DIR "/document_type", NULL);
     if (document_type) {
         set_document_hint (ui, document_type);
         g_free (document_type);
     }
 
-    ui->priv->book_view = book_view_new ();
+    ui->priv->book_view = book_view_new (ui->priv->book);
+    gtk_container_set_border_width (GTK_CONTAINER (ui->priv->book_view), 18);
+    gtk_box_pack_end (GTK_BOX (ui->priv->main_vbox), GTK_WIDGET (ui->priv->book_view), TRUE, TRUE, 0);
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
     g_signal_connect (ui->priv->book_view, "show-page", G_CALLBACK (show_page_cb), ui);
-    book_view_set_widgets (ui->priv->book_view,
-                           ui->priv->preview_box,
-                           ui->priv->preview_area,
-                           ui->priv->preview_scroll,
-                           GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
+    g_signal_connect (ui->priv->book_view, "show-menu", G_CALLBACK (show_page_menu_cb), ui);
+    gtk_widget_show (GTK_WIDGET (ui->priv->book_view));
 
     /* Find default page details */
     scan_direction = gconf_client_get_string(ui->priv->client, GCONF_DIR "/scan_direction", NULL);
@@ -1320,6 +1510,7 @@ ui_load (SimpleScan *ui)
         for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
         if (orientation_keys[i].key != NULL)
             ui->priv->default_page_orientation = orientation_keys[i].orientation;
+        g_free (scan_direction);
     }
     ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_width", NULL);
     if (ui->priv->default_page_width <= 0)
@@ -1348,7 +1539,6 @@ ui_load (SimpleScan *ui)
 
     if (book_get_n_pages (ui->priv->book) == 0)
         add_default_page (ui);
-    book_view_set_book (ui->priv->book_view, ui->priv->book);
 }
 
 
@@ -1362,7 +1552,7 @@ ui_new ()
 Book *
 ui_get_book (SimpleScan *ui)
 {
-    return ui->priv->book;
+    return g_object_ref (ui->priv->book);
 }
 
 
@@ -1393,28 +1583,13 @@ ui_set_scanning (SimpleScan *ui, gboolean scanning)
 void
 ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text, gboolean change_scanner_hint)
 {
-    GtkWidget *dialog;
-
-    dialog = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
-                                     GTK_DIALOG_MODAL,
-                                     GTK_MESSAGE_WARNING,
-                                     GTK_BUTTONS_NONE,
-                                     "%s", error_title);
-    if (change_scanner_hint)
-        gtk_dialog_add_button (GTK_DIALOG (dialog),
-                               /* Button in error dialog to open prefereces dialog and change scanner */
-                               _("Change _Scanner"),
-                               1);
-    gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, 0);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                              "%s", error_text);
-
-    if (gtk_dialog_run (GTK_DIALOG (dialog)) == 1) {
-        gtk_widget_grab_focus (ui->priv->device_combo);
-        gtk_window_present (GTK_WINDOW (ui->priv->preferences_dialog));        
-    }
-
-    gtk_widget_destroy (dialog);
+    ui->priv->have_error = TRUE;
+    g_free (ui->priv->error_title);
+    ui->priv->error_title = g_strdup (error_title);
+    g_free (ui->priv->error_text);
+    ui->priv->error_text = g_strdup (error_text);
+    ui->priv->error_change_scanner_hint = change_scanner_hint;
+    update_info_bar (ui);
 }
 
 
@@ -1427,24 +1602,22 @@ ui_start (SimpleScan *ui)
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__STRING_INT_STRING_INT (GClosure     *closure,
-                                                     GValue       *return_value G_GNUC_UNUSED,
-                                                     guint         n_param_values,
-                                                     const GValue *param_values,
-                                                     gpointer      invocation_hint G_GNUC_UNUSED,
-                                                     gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_POINTER (GClosure     *closure,
+                                              GValue       *return_value G_GNUC_UNUSED,
+                                              guint         n_param_values,
+                                              const GValue *param_values,
+                                              gpointer      invocation_hint G_GNUC_UNUSED,
+                                              gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_INT) (gpointer       data1,
-                                                            gconstpointer  arg_1,
-                                                            gint           arg_2,
-                                                            gconstpointer  arg_3,
-                                                            gint           arg_4,
-                                                            gpointer       data2);
-  register GMarshalFunc_VOID__STRING_INT_STRING_INT callback;
+  typedef void (*GMarshalFunc_VOID__STRING_POINTER) (gpointer       data1,
+                                                     gconstpointer  arg_1,
+                                                     gconstpointer  arg_2,
+                                                     gpointer       data2);
+  register GMarshalFunc_VOID__STRING_POINTER callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 5);
+  g_return_if_fail (n_param_values == 3);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -1456,28 +1629,48 @@ g_cclosure_user_marshal_VOID__STRING_INT_STRING_INT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_INT_STRING_INT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_POINTER) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
-            g_value_get_int (param_values + 2),
-            g_value_get_string (param_values + 3),
-            g_value_get_int (param_values + 4),
+            g_value_get_pointer (param_values + 2),
             data2);
 }
 
 
+static void
+ui_finalize (GObject *object)
+{
+    SimpleScan *ui = SIMPLE_SCAN (object);
+
+    g_object_unref (ui->priv->client);
+    ui->priv->client = NULL;
+    g_object_unref (ui->priv->builder);
+    ui->priv->builder = NULL;
+    g_object_unref (ui->priv->book);
+    ui->priv->book = NULL;
+    gtk_widget_destroy (GTK_WIDGET (ui->priv->book_view));
+    ui->priv->book_view = NULL;
+
+    G_OBJECT_CLASS (ui_parent_class)->finalize (object);
+}
+
+
 static void
 ui_class_init (SimpleScanClass *klass)
 {
+    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+    object_class->finalize = ui_finalize;
+
     signals[START_SCAN] =
         g_signal_new ("start-scan",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_INT_STRING_INT,
-                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
+                      g_cclosure_user_marshal_VOID__STRING_POINTER,
+                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -1492,8 +1685,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, save),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__POINTER,
-                      G_TYPE_NONE, 1, G_TYPE_POINTER);
+                      g_cclosure_marshal_VOID__STRING,
+                      G_TYPE_NONE, 1, G_TYPE_STRING);
     signals[EMAIL] =
         g_signal_new ("email",
                       G_TYPE_FROM_CLASS (klass),
@@ -1527,6 +1720,7 @@ ui_init (SimpleScan *ui)
     ui->priv->client = gconf_client_get_default();
     gconf_client_add_dir(ui->priv->client, GCONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
+    ui->priv->document_hint = g_strdup ("photo");
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
     ui_load (ui);   
